"""
    TOC directive
    ~~~~~~~~~~~~~

    The TOC directive syntax looks like::

        .. toc:: Title
           :level: 3

    "Title" and "level" option can be empty. "level" is an integer less
    than 6, which defines the max heading level writers want to include
    in TOC.
"""

from .base import Directive, parse_options
from ..toc import normalize_toc_item, render_toc_ul


class DirectiveToc(Directive):
    def __init__(self, level=3, heading_id=None):
        self.level = level

        if callable(heading_id):
            self.heading_id = heading_id
        else:
            def heading_id(token, index):
                return 'toc_' + str(index + 1)
            self.heading_id = heading_id

    def parse(self, block, m, state):
        title = m.group('value')
        level = None
        collapse = False
        options = parse_options(m)
        if options:
            d_options = dict(options)
            collapse = 'collapse' in d_options
            level = d_options.get('level')
            if level:
                try:
                    level = int(level)
                except (ValueError, TypeError):
                    return {
                        'type': 'block_error',
                        'raw': 'TOC level MUST be integer',
                    }

        if level is None:
            level = self.level
        elif level < 1 or level > 6:
            level = self.level

        attrs = {'title': title, 'level': level, 'collapse': collapse}
        return {'type': 'toc', 'raw': '', 'attrs': attrs}

    def toc_hook(self, md, state):
        sections = []
        headings = []

        for tok in state.tokens:
            if tok['type'] == 'toc':
                sections.append(tok)
            elif tok['type'] == 'heading':
                headings.append(tok)

        if sections:
            toc_items = []
            # adding ID for each heading
            for i, tok in enumerate(headings):
                tok['attrs']['id'] = self.heading_id(tok, i)
                toc_items.append(normalize_toc_item(md, tok))

            for sec in sections:
                level = sec['attrs']['level']
                toc = [item for item in toc_items if item[0] <= level]
                sec['raw'] = render_toc_ul(toc)

    def __call__(self, md):
        if md.renderer and md.renderer.NAME == 'html':
            # only works with HTML renderer
            self.register_directive(md, 'toc')
            md.before_render_hooks.append(self.toc_hook)
            md.renderer.register('toc', render_html_toc)


def render_html_toc(renderer, text, title, level, collapse=False):
    if not title:
        title = 'Table of Contents'

    html = '<details class="toc"'
    if not collapse:
        html += ' open'
    html += '>\n<summary>' + title + '</summary>\n'
    return html + text + '</details>\n'



"""
    Directive Syntax
    ~~~~~~~~~~~~~~~~~

    This syntax is inspired by reStructuredText. The syntax is very powerful,
    that you can define a lot of custom features by your own.

    The syntax looks like::

        .. directive-name:: directive value
           :option-key: option value
           :option-key: option value

           full featured markdown text here

    :copyright: (c) Hsiaoming Yang
"""

import re

__all__ = ['Directive']


_directive_re = re.compile(
    r'\.\.( +)(?P<name>[a-zA-Z0-9_-]+)\:\: *(?P<value>[^\n]*)(?:\n|$)'
    r'(?P<options>(?:  \1 {0,3}\:[a-zA-Z0-9_-]+\: *[^\n]*\n+)*)'
    r'\n*(?P<text>(?:  \1 {0,3}[^\n]*\n+)*)'
)


class Directive(object):

    def register_directive(self, md, name):
        plugin = getattr(md, '_rst_directive', None)
        if not plugin:
            plugin = PluginDirective()
            plugin(md)

        plugin.register_directive(name, self.parse)

    def parse(self, block, m, state):
        raise NotImplementedError()

    def __call__(self, md):
        raise NotImplementedError()


class PluginDirective(object):
    def __init__(self):
        self._directives = {}

    def register_directive(self, name, fn):
        self._directives[name] = fn

    def parse_block_directive(self, block, m, state):
        m = _directive_re.match(state.src, state.cursor)
        if not m:
            return

        name = m.group('name')
        method = self._directives.get(name)
        if method:
            token = method(block, m, state)
        else:
            token = {
                'type': 'block_error',
                'raw': 'Unsupported directive: ' + name,
            }

        if isinstance(token, list):
            for tok in token:
                state.append_token(tok)
        else:
            state.append_token(token)
        return m.end()

    def __call__(self, md):
        md._rst_directive = self
        pattern = r'^\.\. +[a-zA-Z0-9_-]+\:\:'
        md.block.register('directive', pattern, self.parse_block_directive)


def parse_options(m):
    text = m.group('options')
    if not text.strip():
        return []

    options = []
    for line in re.split(r'\n+', text):
        line = line.strip()[1:]
        if not line:
            continue
        i = line.find(':')
        k = line[:i]
        v = line[i + 1:].strip()
        options.append((k, v))
    return options


def parse_children(block, m, state):
    full_content = m.group(0)
    text = m.group('text')

    pretext = full_content[:-len(text)]

    rules = list(block.rules)
    rules.remove('directive')

    # scan children state
    leading = len(m.group(1)) + 2
    text = '\n'.join(line[leading:] for line in text.splitlines()) + '\n'

    child = state.child_state(text)
    block.parse(child, rules)
    return child.tokens



from .base import Directive, parse_options, parse_children
from .admonition import Admonition
from .toc import DirectiveToc
from .include import DirectiveInclude


__all__ = [
    'parse_options', 'parse_children',
    'Directive',
    'Admonition',
    'DirectiveToc',
    'DirectiveInclude',
]



from .base import Directive, parse_options, parse_children


class Admonition(Directive):
    SUPPORTED_NAMES = {
        "attention", "caution", "danger", "error", "hint",
        "important", "note", "tip", "warning",
    }

    def parse(self, block, m, state):
        options = parse_options(m)

        name = m.group('name')
        title = m.group('value')
        attrs = {'name': name, 'title': title, 'options': options}

        children = parse_children(block, m, state)
        return {
            'type': 'admonition',
            'children': children,
            'attrs': attrs,
        }

    def __call__(self, md):
        for name in self.SUPPORTED_NAMES:
            self.register_directive(md, name)

        if md.renderer.NAME == 'html':
            md.renderer.register('admonition', render_admonition)


def render_admonition(self, text, name, title="", options=None):
    html = '<section class="admonition ' + name + '">\n'
    if not title:
        title = name.capitalize()
    if title:
        html += '<p class="admonition-title">' + title + '</p>\n'
    if text:
        html += text
    return html + '</section>\n'
